@@ -2,7 +2,8 @@
 
 ;; Copyright (C) 1985, 1987, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
 ;;   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-;;   2010, 2011, 2012, 2013   Free Software Foundation, Inc.
+;;   2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
+;;   Free Software Foundation, Inc.
 
 ;; Authors:    2004- Alan Mackenzie
 ;;	       1998- Martin Stjernholm
@@ -23,7 +24,7 @@
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
@@ -55,25 +56,25 @@
 (defun c-lineup-topmost-intro-cont (langelem)
   "Line up declaration continuation lines zero or one indentation step.
 For lines in the \"header\" of a definition, zero is used.  For other
-lines, `c-basic-offset' is added to the indentation.  E.g:
+lines, `c-basic-offset' is added to the indentation.  E.g.:
 
 int
-neg (int i)	      <- c-lineup-topmost-intro-cont
+neg (int i)           <- c-lineup-topmost-intro-cont
 {
     return -i;
 }
 
 struct
-larch		      <- c-lineup-topmost-intro-cont
+larch                 <- c-lineup-topmost-intro-cont
 {
     double height;
 }
-    the_larch,	      <- c-lineup-topmost-intro-cont
+    the_larch,        <- c-lineup-topmost-intro-cont
     another_larch;    <- c-lineup-topmost-intro-cont
 <--> c-basic-offset
 
 struct larch
-the_larch,	      <- c-lineup-topmost-intro-cont
+the_larch,            <- c-lineup-topmost-intro-cont
     another_larch;    <- c-lineup-topmost-intro-cont
 
 \(This function is mainly provided to mimic the behavior of CC Mode
@@ -92,26 +93,26 @@ Works with: topmost-intro-cont."
 
 (defun c-lineup-gnu-DEFUN-intro-cont (langelem)
   "Line up the continuation lines of a DEFUN macro in the Emacs C source.
-These lines are indented as though they were `knr-argdecl-intro' lines.
+These lines are indented `c-basic-offset' columns, usually from column 0.
 Return nil when we're not in such a construct.
 
-This function is for historical compatibility with how previous CC Modes (5.28
-and earlier) indented such lines.
+This function was formally for use in DEFUNs, which used to have knr
+argument lists.  Now (2019-05) it just indents the argument list of the
+DEFUN's function, which would otherwise go to column 0.
 
 Here is an example:
 
 DEFUN (\"forward-char\", Fforward_char, Sforward_char, 0, 1, \"p\",
        doc: /* Move point right N characters (left if N is negative).
 On reaching end of buffer, stop and signal error.  */)
-     (n)		      <- c-lineup-gnu-DEFUN-into-cont
-     Lisp_Object n;	      <- c-lineup-gnu-DEFUN-into-cont
+  (Lisp_Object n)             <- c-lineup-gnu-DEFUN-into-cont
 
 Works with: topmost-intro-cont."
   (save-excursion
     (let (case-fold-search)
       (goto-char (c-langelem-pos langelem))
       (if (looking-at "\\<DEFUN\\>")
-	  (c-calc-offset '(knr-argdecl-intro))))))
+	  c-basic-offset))))
 
 (defun c-block-in-arglist-dwim (arglist-start)
   ;; This function implements the DWIM to avoid far indentation of
@@ -121,7 +122,7 @@ Works with: topmost-intro-cont."
 
   (or
    ;; Check if the syntactic context contains any of the symbols for
-   ;; in-expression constructs.	 This can both save the work that we
+   ;; in-expression constructs.  This can both save the work that we
    ;; have to do below, and it also detect the brace list constructs
    ;; that `c-looking-at-inexpr-block' currently misses (they are
    ;; recognized by `c-inside-bracelist-p' instead).
@@ -167,10 +168,10 @@ Works with: topmost-intro-cont."
 As a special case, if the indented line is inside a brace block
 construct, the indentation is `c-basic-offset' only.  This is intended
 as a \"DWIM\" measure in cases like macros that contains statement
-blocks, e.g:
+blocks, e.g.:
 
 A_VERY_LONG_MACRO_NAME ({
-	some (code, with + long, lines * in[it]);
+        some (code, with + long, lines * in[it]);
     });
 <--> c-basic-offset
 
@@ -186,7 +187,7 @@ Works with: arglist-cont-nonempty, arglist-close."
       (if (c-block-in-arglist-dwim (c-langelem-2nd-pos c-syntactic-element))
 	  c-basic-offset		; DWIM case.
 
-	;; Normal case.	 Indent to the token after the arglist open paren.
+	;; Normal case.  Indent to the token after the arglist open paren.
 	(goto-char (c-langelem-2nd-pos c-syntactic-element))
 	(if (and c-special-brace-lists
 		 (c-looking-at-special-brace-list))
@@ -206,7 +207,7 @@ Works with: arglist-cont-nonempty, arglist-close."
   "Line up a continued argument.
 
 foo (xyz, aaa + bbb + ccc
-	  + ddd + eee + fff);	 <- c-lineup-argcont
+          + ddd + eee + fff);    <- c-lineup-argcont
 
 Only continuation lines like this are touched, nil is returned on lines
 which are the start of an argument.
@@ -288,7 +289,7 @@ the open paren.
 
 As a special case, if a brace block construct starts at the same line
 as the open parenthesis of the argument list, the indentation is
-`c-basic-offset' only.	See `c-lineup-arglist' for further discussion
+`c-basic-offset' only.  See `c-lineup-arglist' for further discussion
 of this \"DWIM\" measure.
 
 Works with: Almost all symbols, but are typically most useful on
@@ -318,8 +319,8 @@ Return nil on lines that don't start with an operator, to leave those
 cases to other line-up functions.  Example:
 
 if (  x < 10
-   || at_limit (x,	 <- c-lineup-arglist-operators
-		list)	 <- c-lineup-arglist-operators returns nil
+   || at_limit (x,       <- c-lineup-arglist-operators
+                list)    <- c-lineup-arglist-operators returns nil
    )
 
 Since this function doesn't do anything for lines without an infix
@@ -329,7 +330,7 @@ suggestion to get a consistent style):
 
 \(c-set-offset 'arglist-cont '(c-lineup-arglist-operators 0))
 \(c-set-offset 'arglist-cont-nonempty '(c-lineup-arglist-operators
-					c-lineup-arglist))
+                                        c-lineup-arglist))
 \(c-set-offset 'arglist-close '(c-lineup-arglist-close-under-paren))
 
 Works with: arglist-cont, arglist-cont-nonempty."
@@ -341,8 +342,8 @@ Works with: arglist-cont, arglist-cont-nonempty."
 
 (defun c-lineup-close-paren (langelem)
   "Line up the closing paren under its corresponding open paren if the
-open paren is followed by code.	 If the open paren ends its line, no
-indentation is added.  E.g:
+open paren is followed by code.  If the open paren ends its line, no
+indentation is added.  E.g.:
 
 main (int,		main (
       char **		    int, char **
@@ -396,19 +397,19 @@ Works with: stream-op."
 
 (defun c-lineup-multi-inher (langelem)
   "Line up the classes in C++ multiple inheritance clauses and member
-initializers under each other.	E.g:
+initializers under each other.  E.g.:
 
-class Foo:		  Foo::Foo (int a, int b):
-    public Cyphr,	      Cyphr (a),
-    public Bar	     <->      Bar (b)		    <- c-lineup-multi-inher
+class Foo:                Foo::Foo (int a, int b):
+    public Cyphr,             Cyphr (a),
+    public Bar       <->      Bar (b)               <- c-lineup-multi-inher
 
-class Foo		  Foo::Foo (int a, int b)
-    : public Cyphr,	      : Cyphr (a),
-      public Bar     <->	Bar (b)		    <- c-lineup-multi-inher
+class Foo                 Foo::Foo (int a, int b)
+    : public Cyphr,           : Cyphr (a),
+      public Bar     <->        Bar (b)             <- c-lineup-multi-inher
 
-class Foo		  Foo::Foo (int a, int b)
-    : public Cyphr	      : Cyphr (a)
-    , public Bar     <->      , Bar (b)		    <- c-lineup-multi-inher
+class Foo                 Foo::Foo (int a, int b)
+    : public Cyphr            : Cyphr (a)
+    , public Bar     <->      , Bar (b)             <- c-lineup-multi-inher
 
 Works with: inher-cont, member-init-cont."
   (save-excursion
@@ -441,11 +442,11 @@ Works with: inher-cont, member-init-cont."
 If class names follow on the same line as the implements/extends
 keyword, they are lined up under each other.  Otherwise, they are
 indented by adding `c-basic-offset' to the column of the keyword.
-E.g:
+E.g.:
 
-class Foo	      class Foo
-    extends		  extends Cyphr,
-	Bar    <->		  Bar	  <- c-lineup-java-inher
+class Foo             class Foo
+    extends               extends Cyphr,
+        Bar    <->                Bar     <- c-lineup-java-inher
     <--> c-basic-offset
 
 Works with: inher-cont."
@@ -463,11 +464,11 @@ If exception names follow on the same line as the throws keyword,
 they are lined up under each other.  Otherwise, they are indented by
 adding `c-basic-offset' to the column of the throws keyword.  The
 throws keyword itself is also indented by `c-basic-offset' from the
-function declaration start if it doesn't hang.	E.g:
+function declaration start if it doesn't hang.  E.g.:
 
-int foo()	    int foo() throws Cyphr,
-    throws     <->		     Bar,    <- c-lineup-java-throws
-	Bar    <->		     Vlod    <- c-lineup-java-throws
+int foo()           int foo() throws Cyphr,
+    throws     <->                   Bar,    <- c-lineup-java-throws
+        Bar    <->                   Vlod    <- c-lineup-java-throws
 <--><--> c-basic-offset
 
 Works with: func-decl-cont."
@@ -487,12 +488,12 @@ Works with: func-decl-cont."
 
 (defun c-indent-one-line-block (langelem)
   "Indent a one line block `c-basic-offset' extra.
-E.g:
+E.g.:
 
-if (n > 0)		   if (n > 0)
-    {m+=n; n=0;}    <->	   {		   <- c-indent-one-line-block
-<--> c-basic-offset	       m+=n; n=0;
-			   }
+if (n > 0)                 if (n > 0)
+    {m+=n; n=0;}    <->    {               <- c-indent-one-line-block
+<--> c-basic-offset            m+=n; n=0;
+                           }
 
 The block may use any kind of parenthesis character.  nil is returned
 if the line doesn't start with a one line block, which makes the
@@ -510,14 +511,14 @@ Work with: Almost all syntactic symbols, but most useful on *-open."
 
 (defun c-indent-multi-line-block (langelem)
   "Indent a multi line block `c-basic-offset' extra.
-E.g:
+E.g.:
 
-int *foo[] = {		 int *foo[] = {
-    NULL,		     NULL,
-    {17},	  <->		 {	 <- c-indent-multi-line-block
-				 17
-				 }
-			     <--> c-basic-offset
+int *foo[] = {           int *foo[] = {
+    NULL,                    NULL,
+    {17},         <->            {       <- c-indent-multi-line-block
+                                 17
+                                 }
+                             <--> c-basic-offset
 
 The block may use any kind of parenthesis character.  nil is returned
 if the line doesn't start with a multi line block, which makes the
@@ -536,11 +537,11 @@ Work with: Almost all syntactic symbols, but most useful on *-open."
 (defun c-lineup-C-comments (langelem)
   "Line up C block comment continuation lines.
 Various heuristics are used to handle many of the common comment
-styles.	 Some examples:
+styles.  Some examples:
 
-/*	    /**		/*	   /* text	/*	    /**
- * text	     * text	  text	      text	** text	     ** text
- */	     */		*/	   */		*/	     */
+/*          /**         /*         /* text      /*          /**
+ * text      * text       text        text      ** text      ** text
+ */          */         */         */           */           */
 
 /*********************************************************************
  * text
@@ -589,7 +590,7 @@ Works with: The `c' syntactic symbol."
 		      1)))))
       (if (and (> starterlen 10) (zerop prefixlen))
 	  ;; The comment has a long starter and the line doesn't have
-	  ;; a nonempty comment prefix.	 Treat it as free form text
+	  ;; a nonempty comment prefix.  Treat it as free form text
 	  ;; and don't change the indentation.
 	  (vector (current-column))
 	;; Go back to the previous non-blank line, if any.
@@ -611,9 +612,9 @@ Works with: The `c' syntactic symbol."
 		  ;; comment-start-skip should match everything (i.e.
 		  ;; typically whitespace) that leads up to the text.
 		  ;;(if (looking-at "\\([ \t]+\\).+$")
-		  ;;	;; Align with the text that hangs after the
-		  ;;	;; comment starter.
-		  ;;	(goto-char (match-end 1)))
+		  ;;    ;; Align with the text that hangs after the
+		  ;;    ;; comment starter.
+		  ;;    (goto-char (match-end 1)))
 		  (vector (current-column)))
 	      ;; How long is the comment starter?  if greater than the
 	      ;; length of the comment prefix, align left.  if less
@@ -625,7 +626,7 @@ Works with: The `c' syntactic symbol."
 		    (vector (1+ (current-column))))
 		(goto-char (+ (c-langelem-pos langelem) starterlen 1))
 		(vector (- (current-column) prefixlen))))
-	  ;; We didn't start on the first non-blank continuation line.	If the
+	  ;; We didn't start on the first non-blank continuation line.  If the
 	  ;; previous line has a nonempty comment prefix, align with it.
 	  ;; Otherwise, align with the previous nonempty line, but align the
 	  ;; comment ender with the starter.
@@ -672,7 +673,7 @@ Works with: comment-intro."
 (defun c-lineup-knr-region-comment (langelem)
   "Line up a comment in the \"K&R region\" with the declaration.
 That is the region between the function or class header and the
-beginning of the block.	 E.g:
+beginning of the block.  E.g.:
 
 int main()
 /* This is the main function. */  <- c-lineup-knr-region-comment
@@ -695,11 +696,11 @@ Works with: comment-intro."
 
 (defun c-lineup-runin-statements (langelem)
   "Line up statements when the first statement is on the same line as
-the block opening brace.  E.g:
+the block opening brace.  E.g.:
 
 int main()
 { puts (\"Hello world!\");
-  return 0;		    <- c-lineup-runin-statements
+  return 0;                 <- c-lineup-runin-statements
 }
 
 If there is no statement after the opening brace to align with, nil is
@@ -717,8 +718,8 @@ Works with: The `statement' syntactic symbol."
 
 (defun c-lineup-assignments (langelem)
   "Line up the current line after the assignment operator on the first
-line in the statement.	If there isn't any, return nil to allow
-stacking with other line-up functions.	If the current line contains
+line in the statement.  If there isn't any, return nil to allow
+stacking with other line-up functions.  If the current line contains
 an assignment operator too, try to align it with the first one.
 
 Works with: topmost-intro-cont, statement-cont, arglist-cont,
@@ -742,7 +743,7 @@ arglist-cont-nonempty."
 	    endpos (c-point 'bol))
 
       ;; Find a syntactically relevant and unnested "=" token on the
-      ;; current line.	equalp is in that case set to the number of
+      ;; current line.  equalp is in that case set to the number of
       ;; columns to left shift the current line to align it with the
       ;; goal column.
       (save-excursion
@@ -796,11 +797,11 @@ arglist-cont-nonempty."
   "Line up \"cascaded calls\" under each other.
 If the line begins with \"->\" or \".\" and the preceding line ends
 with one or more function calls preceded by the same token, then the
-arrow is lined up with the first of those tokens.  E.g:
+arrow is lined up with the first of those tokens.  E.g.:
 
 result = proc->add(17)->add(18)
-	     ->add(19) +	   <- c-lineup-cascaded-calls
-  offset;			   <- c-lineup-cascaded-calls (inactive)
+             ->add(19) +           <- c-lineup-cascaded-calls
+  offset;                          <- c-lineup-cascaded-calls (inactive)
 
 In any other situation nil is returned to allow use in list
 expressions.
@@ -841,10 +842,10 @@ arglist-cont-nonempty."
 (defun c-lineup-string-cont (langelem)
   "Line up a continued string under the one it continues.
 A continued string in this sense is where a string literal follows
-directly after another one.  E.g:
+directly after another one.  E.g.:
 
 result = prefix + \"A message \"
-		  \"string.\";	    <- c-lineup-string-cont
+                  \"string.\";      <- c-lineup-string-cont
 
 In other situations, returns nil, to allow stacking with other
 line-up functions.
@@ -870,12 +871,11 @@ returned if there's no template argument on the first line.
 
 Works with: template-args-cont."
   (save-excursion
-    (c-with-syntax-table c++-template-syntax-table
-      (beginning-of-line)
-      (backward-up-list 1)
-      (if (and (eq (char-after) ?<)
-	       (zerop (c-forward-token-2 1 nil (c-point 'eol))))
-	  (vector (current-column))))))
+    (beginning-of-line)
+    (backward-up-list 1)
+    (if (and (eq (char-after) ?<)
+	     (zerop (c-forward-token-2 1 nil (c-point 'eol))))
+	(vector (current-column)))))
 
 (defun c-lineup-ObjC-method-call (langelem)
   "Line up selector args as Emacs Lisp mode does with function args:
@@ -1025,13 +1025,13 @@ Works with: inlambda, inexpr-statement, inexpr-class."
 (defun c-lineup-whitesmith-in-block (langelem)
   "Line up lines inside a block in Whitesmith style.
 It's done in a way that works both when the opening brace hangs and
-when it doesn't.  E.g:
+when it doesn't.  E.g.:
 
 something
-    {		     something {
-    foo;     <->	 foo;	  <- c-lineup-whitesmith-in-block
-    }			 }
-		     <--> c-basic-offset
+    {                something {
+    foo;     <->         foo;     <- c-lineup-whitesmith-in-block
+    }                    }
+                     <--> c-basic-offset
 
 In the first case the indentation is kept unchanged, in the
 second `c-basic-offset' is added.
@@ -1054,12 +1054,12 @@ inside blocks, this function is necessary for those lines to get
 correct Whitesmith style indentation.  Consider the following
 examples:
 
-		    int foo()
-			{
-int foo()		    {
-    {			    a;
-    a;			    }
-    x;	     <->	x;	  <- c-lineup-after-whitesmith-blocks
+                    int foo()
+                        {
+int foo()                   {
+    {                       a;
+    a;                      }
+    x;       <->        x;        <- c-lineup-after-whitesmith-blocks
 
 The fact that the line with \"x\" is preceded by a Whitesmith style
 indented block in one case and not the other should not affect its
@@ -1069,7 +1069,7 @@ would in the rightmost case be indented too much if the offset for
 `statement' was set simply to zero.
 
 This lineup function corrects for this situation by detecting if the
-anchor position is at an open paren character.	In that case, it
+anchor position is at an open paren character.  In that case, it
 instead indents relative to the surrounding block just like
 `c-lineup-whitesmith-in-block'.
 
@@ -1086,19 +1086,143 @@ arglist-cont."
 	      (vector (+ (current-column) c-basic-offset))))
 	(vector 0)))))
 
+(defun c-lineup-2nd-brace-entry-in-arglist (langelem)
+  "Lineup the second entry of a brace block under the first, when the first
+line is also contained in an arglist or an enclosing brace ON THAT LINE.
+
+I.e. handle something like the following:
+
+    set_line (line_t {point_t{0.4, 0.2},
+                      point_t{0.2, 0.5},       <---- brace-list-intro
+                      .....});
+             ^ enclosing parenthesis.
+
+The middle line of that example will have a syntactic context
+with three syntactic symbols, arglist-cont-nonempty, brace-list-intro, and
+brace-list-entry.
+
+This function is intended for use in a list.  If the construct
+being analyzed isn't like the preceding, the function returns nil.
+Otherwise it returns the function `c-lineup-arglist-intro-after-paren', which
+the caller then uses to perform indentation.
+
+Works with brace-list-intro."
+  ;; brace-list-intro and brace-list-entry are both present for the second
+  ;; entry of the list when the first entry is on the same line as the opening
+  ;; brace.
+  (and (assq 'brace-list-intro c-syntactic-context)
+       (assq 'brace-list-entry c-syntactic-context)
+       (or (assq 'arglist-cont-nonempty c-syntactic-context) ; "(" earlier on
+							     ; the line.
+	   (save-excursion		; "{" earlier on the line
+	     (goto-char (c-langelem-pos
+			 (assq 'brace-list-intro c-syntactic-context)))
+	     (and
+	      (eq (c-backward-token-2
+		   1 nil
+		   (c-point 'bol (c-langelem-pos
+				  (assq 'brace-list-entry
+					c-syntactic-context))))
+		  0)
+	      (eq (char-after) ?{))))
+       'c-lineup-arglist-intro-after-paren))
+
+(defun c-lineup-class-decl-init-+ (langelem)
+  "Line up the second entry of a class (etc.) initializer c-basic-offset
+characters in from the identifier when:
+\(i) The type is a class, struct, union, etc. (but not an enum);
+\(ii) There is a brace block in the type declaration, specifying it; and
+\(iii) The first element of the initializer is on the same line as its opening
+brace.
+
+I.e. we have a construct like this:
+
+    struct STR {
+        int i; float f;
+    } str_1 = {1, 1.7},
+        str_2 = {2,
+             3.1                   <---- brace-list-intro
+        };
+        <-->                       <---- c-basic-offset
+
+Note that the syntactic context of the brace-list-intro line also has a
+syntactic element with the symbol brace-list-entry.
+
+This function is intended for use in a list.  If the above structure isn't
+present, this function returns nil, allowing a different offset specification
+to indent the line.
+
+Works with: brace-list-intro."
+  (and (assq 'brace-list-intro c-syntactic-context)
+       (assq 'brace-list-entry c-syntactic-context)
+       (let ((init-pos (c-point 'boi (c-langelem-pos
+				      (assq 'brace-list-entry
+					    c-syntactic-context))))
+	     )
+	 (save-excursion
+	   (goto-char (c-langelem-pos (assq 'brace-list-intro
+					    c-syntactic-context)))
+	   (and
+	    (c-forward-class-decl)
+	    (not (c-do-declarators init-pos t nil nil nil))
+	    (eq (point) init-pos)
+	    (vector (+ (current-column) c-basic-offset)))))))
+
+(defun c-lineup-class-decl-init-after-brace (langelem)
+  "Line up the second entry of a class (etc.) initializer after its opening
+brace when:
+\(i) The type is a class, struct, union, etc. (but not an enum);
+\(ii) There is a brace block in the type declaration, specifying it; and
+\(iii) The first element of the initializer is on the same line as its opening
+brace.
+
+I.e. we have a construct like this:
+
+    struct STR {
+        int i; float f;
+    } str_1 = {1, 1.7},
+        str_2 = {2,
+                 3.1                   <---- brace-list-intro
+        };
+
+Note that the syntactic context of the brace-list-intro line also has a
+syntactic element with the symbol brace-list-entry.  Also note that this
+function works by returning the symbol `c-lineup-arglist-intro-after-paren',
+which the caller then uses to perform the indentation.
+
+This function is intended for use in a list.  If the above structure isn't
+present, this function returns nil, allowing a different offset specification
+to indent the line.
+
+Works with: brace-list-intro."
+  (and (assq 'brace-list-intro c-syntactic-context)
+       (assq 'brace-list-entry c-syntactic-context)
+       (let ((init-pos (c-point 'boi (c-langelem-pos
+				      (assq 'brace-list-entry
+					    c-syntactic-context))))
+	     )
+	 (save-excursion
+	   (goto-char (c-langelem-pos (assq 'brace-list-intro
+					    c-syntactic-context)))
+	   (and
+	    (c-forward-class-decl)
+	    (not (c-do-declarators init-pos t nil nil nil))
+	    (eq (point) init-pos)
+	    'c-lineup-arglist-intro-after-paren)))))
+
 (defun c-lineup-cpp-define (langelem)
   "Line up macro continuation lines according to the indentation of
-the construct preceding the macro.  E.g:
+the construct preceding the macro.  E.g.:
 
 v beg of preceding constr      v beg of preceding constr
-			     int dribble() {
-const char msg[] =	       if (!running)
-  \"Some text.\";		 error(\"Not running!\");
+                             int dribble() {
+const char msg[] =             if (!running)
+  \"Some text.\";	         error(\"Not running!\");
 
-#define X(A, B)	 \	     #define X(A, B)	\
-do {		 \    <->      do {		\    <- c-lineup-cpp-define
-  printf (A, B); \		 printf (A, B); \
-} while (0)		       } while (0)
+#define X(A, B)  \           #define X(A, B)    \
+do {             \    <->      do {             \    <- c-lineup-cpp-define
+  printf (A, B); \               printf (A, B); \
+} while (0)                    } while (0)
 
 If `c-syntactic-indentation-in-macros' is non-nil, the function
 returns the relative indentation to the macro start line to allow
@@ -1106,14 +1230,14 @@ accumulation with other offsets.  E.g. in the following cases,
 cpp-define-intro is combined with the statement-block-intro that comes
 from the \"do {\" that hangs on the \"#define\" line:
 
-			     int dribble() {
-const char msg[] =	       if (!running)
-  \"Some text.\";		 error(\"Not running!\");
+                             int dribble() {
+const char msg[] =             if (!running)
+  \"Some text.\";	         error(\"Not running!\");
 
-#define X(A, B) do { \	     #define X(A, B) do { \
-  printf (A, B);     \	<->	 printf (A, B);	  \  <- c-lineup-cpp-define
-  this->refs++;	     \		 this->refs++;	  \
-} while (0)		<->    } while (0)	     <- c-lineup-cpp-define
+#define X(A, B) do { \       #define X(A, B) do { \
+  printf (A, B);     \  <->      printf (A, B);   \  <- c-lineup-cpp-define
+  this->refs++;      \           this->refs++;    \
+} while (0)             <->    } while (0)           <- c-lineup-cpp-define
 
 The relative indentation returned by `c-lineup-cpp-define' is zero and
 two, respectively, in these two examples.  They are then added to the
@@ -1173,17 +1297,17 @@ Works with: cpp-define-intro."
   "Line up a gcc asm register under one on a previous line.
 
     asm (\"foo %1, %0\\n\"
-	 \"bar %0, %1\"
-	 : \"=r\" (w),
-	   \"=r\" (x)
-	 :  \"0\" (y),
-	    \"1\" (z));
+         \"bar %0, %1\"
+         : \"=r\" (w),
+           \"=r\" (x)
+         :  \"0\" (y),
+            \"1\" (z));
 
 The \"x\" line is aligned to the text after the \":\" on the \"w\" line, and
 similarly \"z\" under \"y\".
 
 This is done only in an \"asm\" or \"__asm__\" block, and only to
-those lines mentioned.	Anywhere else nil is returned.	The usual
+those lines mentioned.  Anywhere else nil is returned.  The usual
 arrangement is to have this routine as an extra feature at the start
 of arglist line-ups, e.g.
 
@@ -1223,6 +1347,18 @@ Works with: arglist-cont, arglist-cont-nonempty."
 
        (vector (progn (goto-char alignto) (current-column)))))))
 
+(defun c-lineup-under-anchor (langelem)
+  "Line up the current line directly under the anchor position in LANGELEM.
+
+This is like 0, except it supersedes any indentation already calculated for
+previous syntactic elements in the syntactic context.
+
+Works with: Any syntactic symbol which has an anchor position."
+  (save-excursion
+    (goto-char (c-langelem-pos langelem))
+    (vector (current-column))))
+    
+
 (defun c-lineup-dont-change (langelem)
   "Do not change the indentation of the current line.
 
@@ -1232,7 +1368,7 @@ Works with: Any syntactic symbol."
     (vector (current-column))))
 
 (defun c-lineup-respect-col-0 (langelem)
-  "If the current line starts at column 0, return [0].	Otherwise return nil.
+  "If the current line starts at column 0, return [0].  Otherwise return nil.
 
 This can be used for comments (in conjunction with, say,
 `c-lineup-comment'), to keep comments already at column 0
@@ -1264,7 +1400,7 @@ ACTION associated with `block-close' syntax."
 
 (defun c-snug-1line-defun-close (syntax pos)
   "Determine the brace hanginess for an AWK defun-close.
-If the action/function being closed is a one-liner, keep it so.	 Otherwise put
+If the action/function being closed is a one-liner, keep it so.  Otherwise put
 the closing brace on its own line."
   (save-excursion
     (goto-char pos)
@@ -1296,7 +1432,7 @@ indentation amount."
   "Controls newline insertion after semicolons in parenthesis lists.
 If a comma was inserted, no determination is made.  If a semicolon was
 inserted inside a parenthesis list, no newline is added otherwise a
-newline is added.  In either case, checking is stopped.	 This supports
+newline is added.  In either case, checking is stopped.  This supports
 exactly the old newline insertion behavior."
   ;; newline only after semicolon, but only if that semicolon is not
   ;; inside a parenthesis list (e.g. a for loop statement)
@@ -1319,7 +1455,7 @@ Otherwise, no determination is made."
   (save-excursion
     (if (and (= (c-last-command-char) ?\;)
 	     ;;(/= (point-max)
-	     ;;	   (save-excursion (skip-syntax-forward " ") (point))
+	     ;;    (save-excursion (skip-syntax-forward " ") (point))
 	     (zerop (forward-line 1))
 	     (bolp)			; forward-line has funny behavior at eob.
 	     (not (looking-at "^[ \t]*$")))
@@ -1347,5 +1483,8 @@ For other semicolon contexts, no determination is made."
 
 (cc-provide 'cc-align)
 
-;;; arch-tag: 4d71ed28-bf51-4509-a148-f39669669a2e
+;;; Local Variables:
+;;; indent-tabs-mode: t
+;;; tab-width: 8
+;;; End:
 ;;; cc-align.el ends here
