@@ -1,7 +1,8 @@
 ;;; cc-guess.el --- guess indentation values by scanning existing code
 
 ;; Copyright (C) 1985,1987,1992-2003, 2004, 2005, 2006, 2007, 2008,
-;; 2009, 2010, 2011, 2012, 2013  Free Software Foundation, Inc.
+;; 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
+;; Free Software Foundation, Inc.
 
 ;; Author:     1994-1995 Barry A. Warsaw
 ;;             2011-     Masatake YAMATO
@@ -523,7 +524,8 @@ is called with one argument, the guessed style."
       (goto-char (point-min))
       (when (search-forward (concat "("
 				    (symbol-name (car needs-markers))
-				    " ") nil t)
+				    " ")
+			    nil t)
 	(move-end-of-line 1)
 	(comment-dwim nil)
 	(insert " Guessed value"))
@@ -575,4 +577,9 @@ WITH-NAME is asked to the user."
 
 
 (cc-provide 'cc-guess)
+
+;;; Local Variables:
+;;; indent-tabs-mode: t
+;;; tab-width: 8
+;;; End:
 ;;; cc-guess.el ends here
