@@ -2,7 +2,8 @@
 
 ;; Copyright (C) 1985, 1987, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
 ;;   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-;;   2010, 2011, 2012, 2013   Free Software Foundation, Inc.
+;;   2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
+;;   Free Software Foundation, Inc.
 
 ;; Authors:    2004- Alan Mackenzie
 ;;	       1998- Martin Stjernholm
@@ -23,7 +24,7 @@
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
@@ -49,6 +50,7 @@
 ;; `c-add-style' often contains references to functions defined there.
 
 ;; Silence the compiler.
+(cc-bytecomp-defun c-guess-basic-syntax)
 (cc-bytecomp-defvar adaptive-fill-first-line-regexp) ; Emacs
 (cc-bytecomp-obsolete-fun make-local-hook) ; Marked obsolete in Emacs 21.1.
 					   ; Allegedly still required by
@@ -59,7 +61,7 @@
   '(("gnu"
      (c-basic-offset . 2)
      (c-comment-only-line-offset . (0 . 0))
-     (c-hanging-braces-alist	 . ((substatement-open before after)
+     (c-hanging-braces-alist     . ((substatement-open before after)
 				    (arglist-cont-nonempty)))
      (c-offsets-alist . ((statement-block-intro . +)
 			 (knr-argdecl-intro . 5)
@@ -72,6 +74,9 @@
 			 (arglist-close . c-lineup-arglist)
 			 (inline-open . 0)
 			 (brace-list-open . +)
+			 (brace-list-intro . (first
+					      c-lineup-2nd-brace-entry-in-arglist
+					      c-lineup-class-decl-init-+ +))
 			 (topmost-intro-cont
 			  . (first c-lineup-topmost-intro-cont
 				   c-lineup-gnu-DEFUN-intro-cont))))
@@ -98,6 +103,9 @@
 			 (label . 0)
 			 (statement-cont . +)
 			 (inline-open . 0)
+			 (brace-list-intro . (first
+					      c-lineup-2nd-brace-entry-in-arglist
+					      c-lineup-class-decl-init-+ +))
 			 (inexpr-class . 0))))
 
     ("stroustrup"
@@ -107,6 +115,9 @@
 			 (substatement-open . 0)
 			 (substatement-label . 0)
 			 (label . 0)
+			 (brace-list-intro . (first
+					      c-lineup-2nd-brace-entry-in-arglist
+					      c-lineup-class-decl-init-+ +))
 			 (statement-cont . +))))
 
     ("whitesmith"
@@ -165,15 +176,15 @@
     ("ellemtel"
      (c-basic-offset . 3)
      (c-comment-only-line-offset . 0)
-     (c-hanging-braces-alist	 . ((substatement-open before after)
+     (c-hanging-braces-alist     . ((substatement-open before after)
 				    (arglist-cont-nonempty)))
-     (c-offsets-alist . ((topmost-intro	       . 0)
-			 (substatement	       . +)
+     (c-offsets-alist . ((topmost-intro        . 0)
+			 (substatement         . +)
 			 (substatement-open    . 0)
-			 (case-label	       . +)
-			 (access-label	       . -)
-			 (inclass	       . +)
-			 (inline-open	       . 0))))
+			 (case-label           . +)
+			 (access-label         . -)
+			 (inclass              . +)
+			 (inline-open          . 0))))
     ("linux"
      (c-basic-offset  . 8)
      (c-comment-only-line-offset . 0)
@@ -184,11 +195,11 @@
 				(arglist-cont-nonempty)))
      (c-cleanup-list . (brace-else-brace))
      (c-offsets-alist . ((statement-block-intro . +)
-			 (knr-argdecl-intro	. 0)
-			 (substatement-open	. 0)
-			 (substatement-label	. 0)
-			 (label			. 0)
-			 (statement-cont	. +))))
+			 (knr-argdecl-intro     . 0)
+			 (substatement-open     . 0)
+			 (substatement-label    . 0)
+			 (label                 . 0)
+			 (statement-cont        . +))))
 
     ("python"
      (indent-tabs-mode . t)
@@ -197,6 +208,9 @@
      (c-offsets-alist  . ((substatement-open . 0)
 			  (inextern-lang . 0)
 			  (arglist-intro . +)
+			  (brace-list-intro . (first
+					       c-lineup-2nd-brace-entry-in-arglist
+					       c-lineup-class-decl-init-+ +))
 			  (knr-argdecl-intro . +)))
      (c-hanging-braces-alist . ((brace-list-open)
 				(brace-list-intro)
@@ -212,18 +226,21 @@
      (c-comment-only-line-offset . (0 . 0))
      ;; the following preserves Javadoc starter lines
      (c-offsets-alist . ((inline-open . 0)
-			 (topmost-intro-cont	. +)
+			 (topmost-intro-cont    . +)
 			 (statement-block-intro . +)
-			 (knr-argdecl-intro	. 5)
-			 (substatement-open	. +)
-			 (substatement-label	. +)
-			 (label			. +)
-			 (statement-case-open	. +)
-			 (statement-cont	. +)
-			 (arglist-intro	 . c-lineup-arglist-intro-after-paren)
-			 (arglist-close	 . c-lineup-arglist)
-			 (access-label	 . 0)
-			 (inher-cont	 . c-lineup-java-inher)
+			 (knr-argdecl-intro     . 5)
+			 (substatement-open     . +)
+			 (substatement-label    . +)
+			 (label                 . +)
+			 (statement-case-open   . +)
+			 (statement-cont        . +)
+			 (arglist-intro  . c-lineup-arglist-intro-after-paren)
+			 (arglist-close  . c-lineup-arglist)
+			 (brace-list-intro . (first
+					      c-lineup-2nd-brace-entry-in-arglist
+					      c-lineup-class-decl-init-+ +))
+			 (access-label   . 0)
+			 (inher-cont     . c-lineup-java-inher)
 			 (func-decl-cont . c-lineup-java-throws))))
 
     ;; awk style exists primarily for auto-newline settings.  Otherwise it's
@@ -356,9 +373,9 @@ might get set too.
 
 If DONT-OVERRIDE is neither nil nor t, style variables whose default values
 have been set (more precisely, whose default values are not the symbol
-`set-from-style') will not be changed.	This avoids overriding global settings
-done in ~/.emacs.  It is useful to call c-set-style from a mode hook in this
-way.
+`set-from-style') will not be changed.  This avoids overriding global settings
+done in your init file.  It is useful to call c-set-style from a mode hook
+in this way.
 
 If DONT-OVERRIDE is t, style variables that already have values (i.e., whose
 values are not the symbol `set-from-style') will not be overridden.  CC Mode
@@ -583,7 +600,7 @@ CC Mode by making sure the proper entries are present on
   ;; only.
 
   ;; The default configuration already handles C++ comments, but we
-  ;; need to add handling of C block comments.	A new filladapt token
+  ;; need to add handling of C block comments.  A new filladapt token
   ;; `c-comment' is added for that.
   (let (p)
     (setq p filladapt-token-table)
@@ -670,5 +687,8 @@ DEFAULT-STYLE has the same format as `c-default-style'."
 
 (cc-provide 'cc-styles)
 
-;;; arch-tag: c764f61a-96ba-484a-a68f-101c0e9d5d2c
+;;; Local Variables:
+;;; indent-tabs-mode: t
+;;; tab-width: 8
+;;; End:
 ;;; cc-styles.el ends here
